@@ -16,7 +16,7 @@ use nom::{
     combinator::{map, map_res, verify},
     number::{complete::u32, complete::u64, Endianness},
     sequence::{delimited, pair, preceded, terminated},
-    IResult,
+    IResult, Parser,
 };
 
 use super::{
@@ -287,12 +287,13 @@ impl<'de> PodDeserializer<'de> {
 
     /// Parse the size from the header and ensure it has the correct type.
     pub(super) fn header<'b>(type_: u32) -> impl FnMut(&'b [u8]) -> IResult<&'b [u8], u32> {
-        terminated(u32(Endianness::Native), tag(type_.to_ne_bytes()))
+        let bytes = type_.to_ne_bytes();
+        move |input| terminated(u32(Endianness::Native), tag(&bytes[..])).parse(input)
     }
 
     /// Parse and return the type from the header
     pub(super) fn type_<'b>() -> impl FnMut(&'b [u8]) -> IResult<&'b [u8], u32> {
-        preceded(u32(Endianness::Native), u32(Endianness::Native))
+        move |input| preceded(u32(Endianness::Native), u32(Endianness::Native)).parse(input)
     }
 
     /// Deserialize any fixed size pod.
@@ -304,13 +305,16 @@ impl<'de> PodDeserializer<'de> {
     ) -> Result<(P, DeserializeSuccess<'de>), DeserializeError<&'de [u8]>> {
         let padding = Self::calc_padding_needed(P::CanonicalType::SIZE);
 
-        self.parse(delimited(
-            Self::header(P::CanonicalType::TYPE),
-            map(P::CanonicalType::deserialize_body, |res| {
-                P::from_canonical_type(&res)
-            }),
-            take(padding),
-        ))
+        self.parse(move |input| {
+            delimited(
+                Self::header(P::CanonicalType::TYPE),
+                map(P::CanonicalType::deserialize_body, |res| {
+                    P::from_canonical_type(&res)
+                }),
+                take(padding),
+            )
+            .parse(input)
+        })
         .map(|res| (res, DeserializeSuccess(self)))
         .map_err(|err| err.into())
     }
@@ -397,10 +401,18 @@ impl<'de> PodDeserializer<'de> {
     {
         let len = self.parse(Self::header(spa_sys::SPA_TYPE_String))?;
         let padding = Self::calc_padding_needed(len);
-        let res = self.parse(terminated(
-            map_res(terminated(take(len - 1), tag([b'\0'])), std::str::from_utf8),
-            take(padding),
-        ))?;
+
+        let res = self.parse(move |input| {
+            terminated(
+                map_res(
+                    terminated(take(len - 1), tag(&b"\0"[..])),
+                    std::str::from_utf8,
+                ),
+                take(padding),
+            )
+            .parse(input)
+        })?;
+
         Ok((visitor.visit_string(res)?, DeserializeSuccess(self)))
     }
 
@@ -414,7 +426,8 @@ impl<'de> PodDeserializer<'de> {
     {
         let len = self.parse(Self::header(spa_sys::SPA_TYPE_Bytes))?;
         let padding = Self::calc_padding_needed(len);
-        let res = self.parse(terminated(take(len), take(padding)))?;
+        let res = self.parse(move |input| terminated(take(len), take(padding)).parse(input))?;
+
         Ok((visitor.visit_bytes(res)?, DeserializeSuccess(self)))
     }
 
@@ -430,9 +443,12 @@ impl<'de> PodDeserializer<'de> {
         E: FixedSizedPod,
     {
         let len = self.parse(Self::header(spa_sys::SPA_TYPE_Array))?;
-        self.parse(verify(Self::header(E::CanonicalType::TYPE), |len| {
-            *len == E::CanonicalType::SIZE
-        }))?;
+        self.parse(move |input| {
+            verify(Self::header(E::CanonicalType::TYPE), |len| {
+                *len == E::CanonicalType::SIZE
+            })
+            .parse(input)
+        })?;
 
         let num_elems = if E::CanonicalType::SIZE != 0 {
             (len - 8) / E::CanonicalType::SIZE
@@ -474,8 +490,9 @@ impl<'de> PodDeserializer<'de> {
         mut self,
     ) -> Result<ObjectPodDeserializer<'de>, DeserializeError<&'de [u8]>> {
         let len = self.parse(Self::header(spa_sys::SPA_TYPE_Object))?;
-        let (object_type, object_id) =
-            self.parse(pair(u32(Endianness::Native), u32(Endianness::Native)))?;
+        let (object_type, object_id) = self.parse(move |input| {
+            pair(u32(Endianness::Native), u32(Endianness::Native)).parse(input)
+        })?;
 
         Ok(ObjectPodDeserializer {
             deserializer: Some(self),
@@ -626,10 +643,15 @@ impl<'de> PodDeserializer<'de> {
         V: Visitor<'de>,
     {
         let len = self.parse(Self::header(spa_sys::SPA_TYPE_Choice))?;
-        let (choice_type, flags) =
-            self.parse(pair(u32(Endianness::Native), u32(Endianness::Native)))?;
-        let (child_size, child_type) =
-            self.parse(pair(u32(Endianness::Native), u32(Endianness::Native)))?;
+
+        let (choice_type, flags) = self.parse(move |input| {
+            pair(u32(Endianness::Native), u32(Endianness::Native)).parse(input)
+        })?;
+
+        let (child_size, child_type) = self.parse(move |input| {
+            pair(u32(Endianness::Native), u32(Endianness::Native)).parse(input)
+        })?;
+
         let num_values = (len - 16) / child_size;
 
         fn create_choice<'de, E>(
@@ -768,20 +790,21 @@ impl<'de> PodDeserializer<'de> {
         V: Visitor<'de>,
     {
         let len = self.parse(Self::header(spa_sys::SPA_TYPE_Pointer))?;
-        let (type_, _padding) =
-            self.parse(pair(u32(Endianness::Native), u32(Endianness::Native)))?;
+        let (type_, _padding) = self.parse(move |input| {
+            pair(u32(Endianness::Native), u32(Endianness::Native)).parse(input)
+        })?;
         let ptr_size = len - 8;
 
         let res = match ptr_size {
             8 => {
-                let ptr = self.parse(u64(Endianness::Native))?;
+                let ptr: u64 = self.parse(|input| u64(Endianness::Native).parse(input))?;
                 visitor.visit_pointer(type_, ptr as *const c_void)?
             }
             4 => {
-                let ptr = self.parse(u32(Endianness::Native))?;
+                let ptr: u32 = self.parse(|input| u32(Endianness::Native).parse(input))?;
                 visitor.visit_pointer(type_, ptr as *const c_void)?
             }
-            _ => panic!("unsupported pointer size {}", ptr_size),
+            _ => panic!("unsupported pointer size {ptr_size}"),
         };
 
         Ok((res, DeserializeSuccess(self)))
@@ -818,7 +841,8 @@ impl<'de> PodDeserializer<'de> {
     fn deserialize_array_any(
         self,
     ) -> Result<(Value, DeserializeSuccess<'de>), DeserializeError<&'de [u8]>> {
-        let child_type = self.peek(preceded(Self::type_(), Self::type_()))?;
+        let child_type =
+            self.peek(move |input| preceded(Self::type_(), Self::type_()).parse(input))?;
 
         let (array, success) = match child_type {
             spa_sys::SPA_TYPE_None => {
