@@ -115,6 +115,13 @@ pub mod c {
             n_elems: u32,
             elems: *const i64,
         ) -> *const spa_pod;
+        pub fn build_audio_info_raw(
+            buffer: *mut u8,
+            len: usize,
+            id: u32,
+            audio_raw: *const spa_sys::spa_audio_info_raw,
+        ) -> *const spa_pod;
+        pub fn parse_audio_info_raw(val: *const spa_pod) -> i32;
         pub fn build_pointer(
             buffer: *mut u8,
             len: usize,
@@ -2147,6 +2154,8 @@ fn pointer() {
     );
 }
 
+use libspa::param::audio::{self, AudioFormat, AudioInfoRaw};
+
 #[test]
 #[cfg_attr(miri, ignore)]
 fn composite_values() {
@@ -2215,3 +2224,34 @@ fn composite_values() {
         Ok((&[] as &[u8], object_val))
     );
 }
+
+#[test]
+#[cfg_attr(miri, ignore)]
+fn audio_info_raw() {
+    let id = 1;
+    let position = [0; audio::MAX_CHANNELS];
+    let mut info = AudioInfoRaw::new();
+    info.set_channels(1);
+    info.set_rate(44100);
+    info.set_format(AudioFormat::S8);
+    info.set_position(position);
+
+    let obj_rs = Value::Object(Object {
+        type_: spa_sys::SPA_TYPE_OBJECT_Format,
+        id,
+        properties: info.into(),
+    });
+    let vec_rs: Vec<u8> = PodSerializer::serialize(Cursor::new(Vec::new()), &obj_rs)
+        .unwrap()
+        .0
+        .into_inner();
+
+    let mut vec_c: Vec<u8> = vec![0; vec_rs.len()];
+    let obj_c: spa_sys::spa_audio_info_raw = info.as_raw();
+    assert_ne!(
+        unsafe { c::build_audio_info_raw(vec_c.as_mut_ptr(), vec_c.len(), id, &obj_c) },
+        std::ptr::null()
+    );
+    assert_eq!(vec_rs, vec_c);
+    assert!(unsafe { c::parse_audio_info_raw(vec_c.as_mut_ptr()) } > 0);
+}
